#include <QMessageBox>
#include <QDebug>
#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "departmodel.h"
#include "departdelegate.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    ui->dateTimeEdit->setDateTime(QDateTime::currentDateTime());



    depModel = new departModel(this);
    ui->departTableView->setModel(depModel);
    departDelegate * dDelegate = new departDelegate(ui->departTableView);
    ui->departTableView->setItemDelegate(dDelegate);
    ui->departTableView->setEditTriggers(QAbstractItemView::AllEditTriggers);
    ui->departTableView->horizontalHeader()->setResizeMode(QHeaderView::ResizeToContents);
    ui->departTableView->horizontalHeader()->resizeSection(0,ICON_WIDTH);

    timer = new QTimer(this);
    timer->setInterval(1000*60);
    connect(timer,SIGNAL(timeout()),this,SLOT(timerTimeout()));

    rGare = new rechercheGare(this);
    connect(rGare,SIGNAL(resConnu(QList<QPair<QString,QString> >)),this,SLOT(listeGareConnue(QList<QPair<QString,QString> >)));
    connect(rGare,SIGNAL(erreurReseau(QString)),this,SLOT(erreurRechercheGare(QString)));

    dTrains = new departTrains(this);
    connect(dTrains,SIGNAL(departsConnus(QList<ligneDepart>)),this,SLOT(listeDepartsConnus(QList<ligneDepart>)));

    settings = new QSettings("WidgetTER","WidgetTER");
    this->chargerPreferences();
}

MainWindow::~MainWindow()
{
    this->sauverPreferences();
    delete ui;
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void MainWindow::on_recherchePushButton_clicked()
{
    if (ui->rechercheLineEdit->text().isEmpty())
    {
        QMessageBox::warning(this,tr("Recherche de gare"),tr("Veuillez renseigner le champ de recherche."),QMessageBox::Ok);
        return;
    }
    ui->recherchePushButton->setEnabled(false);
    ui->refreshPushButton->setEnabled(false);
    ui->statusBar->showMessage("Recherche gares en cours...");
    this->setCursor(QCursor(Qt::BusyCursor));
    timer->stop();
    rGare->demandeServeur(ui->rechercheLineEdit->text());

}

void MainWindow::listeGareConnue(QList<QPair<QString, QString> >listeGare)
{
    this->statusBar()->clearMessage();
    this->setCursor(QCursor(Qt::ArrowCursor));
    ui->garesComboBox->clear();
    while (!listeGare.isEmpty())
    {
        QPair <QString,QString> pairGare = listeGare.takeFirst();
        ui->garesComboBox->addItem(pairGare.second,pairGare.first);
    }
    ui->recherchePushButton->setEnabled(true);
}

void MainWindow::erreurRechercheGare(QString error)
{
    QMessageBox::critical(this,tr("Recherche de gare"),tr("Erreur réseau.\nDétails:") + error,QMessageBox::Ok);
    ui->recherchePushButton->setEnabled(true);    
    ui->statusBar->clearMessage();
    this->setCursor(QCursor(Qt::ArrowCursor));
}

void MainWindow::on_garesComboBox_currentIndexChanged(int index)
{
    ui->departsGroupBox->setTitle(tr("Trains au départ de: ") + ui->garesComboBox->currentText());
    if (ui->garesComboBox->itemData(index) != QVariant::Invalid)
    {
        idGareCourante = ui->garesComboBox->itemData(index).toString();
        if (!idGareCourante.isEmpty())
        {
            nomGareCourante = ui->garesComboBox->itemText(index);
            QList<ligneDepart> ld;
            depModel->setDepartListe(ld);
            if (ui->nowRadioButton->isChecked())
                dTrains->demandeDeparts(idGareCourante,ui->nbDepSpinBox->value());
            else
                dTrains->demandeDeparts(idGareCourante,ui->nbDepSpinBox->value(),ui->dateTimeEdit->dateTime());
            ui->refreshPushButton->setEnabled(false);
            ui->statusBar->showMessage("Recherche départs en cours...");
            this->setCursor(QCursor(Qt::BusyCursor));
            timer->start(1000*60);
        }
    }

}

void MainWindow::listeDepartsConnus(QList<ligneDepart>listeLignesDeparts)
{    
    //qDebug() << "Nbre de départs: " << listeLignesDeparts.size();
    depModel->setDepartListe(listeLignesDeparts);    
    ui->departTableView->resizeColumnsToContents();
    ui->departTableView->horizontalHeader()->setStretchLastSection(true);
    ui->statusBar->clearMessage();
    this->setCursor(QCursor(Qt::ArrowCursor));
    timer->start(1000*60);
    ui->refreshPushButton->setEnabled(true);
}

void MainWindow::timerTimeout()
{
    if (ui->nowRadioButton->isChecked())
        ui->dateTimeEdit->setDateTime(QDateTime::currentDateTime());
    if (!idGareCourante.isEmpty())
    {
        if (ui->nowRadioButton->isChecked())
            dTrains->demandeDeparts(idGareCourante,ui->nbDepSpinBox->value());
        else
            dTrains->demandeDeparts(idGareCourante,ui->nbDepSpinBox->value(),ui->dateTimeEdit->dateTime());
        ui->refreshPushButton->setEnabled(false);
        ui->statusBar->showMessage("Recherche départs en cours...");
        timer->stop();
        this->setCursor(QCursor(Qt::BusyCursor));
    }
}

void MainWindow::on_rechercheLineEdit_returnPressed()
{
    this->on_recherchePushButton_clicked();
}


void MainWindow::on_nowRadioButton_toggled(bool checked)
{
    if (checked)
        qDebug () << "MainWindow::on_nowRadioButton_toggled";
}

void MainWindow::chargerPreferences()
{    
    ui->nbDepSpinBox->setValue(settings->value("nbDeparts",4).toInt());
    idGareCourante = settings->value("IDGare").toString();
    nomGareCourante = settings->value("NomGare").toString();
    ui->garesComboBox->addItem(nomGareCourante,idGareCourante);
}

void MainWindow::sauverPreferences()
{
    settings->setValue("nbDeparts",ui->nbDepSpinBox->value());
    if (!idGareCourante.isEmpty())
    {
        settings->setValue("IDGare",idGareCourante);
        settings->setValue("NomGare",nomGareCourante);
    }
    settings->sync();
}

void MainWindow::on_refreshPushButton_clicked()
{
    this->timerTimeout();
}
